@@ -67,7 +67,7 @@ def meta_config(meta):
     return config
 
 
-def render_run_docker_build(jinja_env, forge_config, forge_dir):
+def render_circle(jinja_env, forge_config, forge_dir):
     meta = forge_config['package']
     with fudge_subdir('linux-64', build_config=meta_config(meta)):
         meta.parse_again()
@@ -90,8 +90,9 @@ def render_run_docker_build(jinja_env, forge_config, forge_dir):
         forge_config["circle"]["enabled"] = False
 
         target_fnames = [
-            os.path.join(forge_dir, 'ci_support', 'run_docker_build.sh'),
             os.path.join(forge_dir, 'ci_support', 'checkout_merge_commit.sh'),
+            os.path.join(forge_dir, 'ci_support', 'fast_finish_ci_pr_build.sh'),
+            os.path.join(forge_dir, 'ci_support', 'run_docker_build.sh'),
         ]
         for each_target_fname in target_fnames:
             remove_file(each_target_fname)
@@ -100,6 +101,29 @@ def render_run_docker_build(jinja_env, forge_config, forge_dir):
         matrix = prepare_matrix_for_env_vars(matrix)
         forge_config = update_matrix(forge_config, matrix)
 
+        fast_finish = textwrap.dedent("""\
+            {get_fast_finish_script} | \\
+                 python - -v --ci "circle" "${{CIRCLE_PROJECT_USERNAME}}/${{CIRCLE_PROJECT_REPONAME}}" "${{CIRCLE_BUILD_NUM}}" "${{CIRCLE_PR_NUMBER}}"
+        """)
+        get_fast_finish_script = ""
+
+        # If the recipe supplies its own conda-forge-build-setup script,
+        # we use it instead of the global one.
+        cfbs_fpath = os.path.join(forge_dir, 'recipe',
+                                  'ff_ci_pr_build.py')
+        if os.path.exists(cfbs_fpath):
+            get_fast_finish_script += "cat {recipe_dir}/ff_ci_pr_build.py".format(recipe_dir=forge_config["recipe_dir"])
+        else:
+            get_fast_finish_script += "curl https://raw.githubusercontent.com/conda-forge/conda-forge-build-setup-feedstock/master/recipe/ff_ci_pr_build.py"
+
+        fast_finish = fast_finish.format(
+            get_fast_finish_script=get_fast_finish_script
+        )
+
+        fast_finish = fast_finish.strip()
+
+        forge_config['fast_finish'] = fast_finish
+
         build_setup = ""
 
         # If the recipe supplies its own conda-forge-build-setup script,
@@ -157,42 +181,27 @@ def render_run_docker_build(jinja_env, forge_config, forge_dir):
         templates = forge_config.get('templates', {})
         template_name = templates.get('run_docker_build',
                                       'run_docker_build_matrix.tmpl')
-
         template = jinja_env.get_template(template_name)
         target_fname = os.path.join(forge_dir, 'ci_support', 'run_docker_build.sh')
         with write_file(target_fname) as fh:
             fh.write(template.render(**forge_config))
 
+        template_name = 'fast_finish_ci_pr_build.sh.tmpl'
+        template = jinja_env.get_template(template_name)
+        target_fname = os.path.join(forge_dir, 'ci_support', 'fast_finish_ci_pr_build.sh')
+        with write_file(target_fname) as fh:
+            fh.write(template.render(**forge_config))
+
         # Fix permissions.
         target_fnames = [
-            os.path.join(forge_dir, 'ci_support', 'run_docker_build.sh'),
             os.path.join(forge_dir, 'ci_support', 'checkout_merge_commit.sh'),
+            os.path.join(forge_dir, 'ci_support', 'fast_finish_ci_pr_build.sh'),
+            os.path.join(forge_dir, 'ci_support', 'run_docker_build.sh'),
         ]
         for each_target_fname in target_fnames:
             set_exe_file(each_target_fname, True)
 
-
-def render_circle(jinja_env, forge_config, forge_dir):
-    meta = forge_config['package']
-    with fudge_subdir('linux-64', build_config=meta_config(meta)):
-        meta.parse_again()
-        matrix = compute_build_matrix(
-            meta,
-            forge_config.get('matrix'),
-            forge_config.get('channels', {}).get('sources', tuple())
-        )
-
-        cases_not_skipped = []
-        for case in matrix:
-            pkgs, vars = split_case(case)
-            with enable_vars(vars):
-                if not ResolvedDistribution(meta, pkgs).skip():
-                    cases_not_skipped.append(vars + sorted(pkgs))
-        matrix = sorted(cases_not_skipped, key=sort_without_target_arch)
-
     target_fname = os.path.join(forge_dir, 'circle.yml')
-    matrix = prepare_matrix_for_env_vars(matrix)
-    forge_config = update_matrix(forge_config, matrix)
     template = jinja_env.get_template('circle.yml.tmpl')
     with write_file(target_fname) as fh:
         fh.write(template.render(**forge_config))
@@ -259,6 +268,30 @@ def render_travis(jinja_env, forge_config, forge_dir):
         matrix = prepare_matrix_for_env_vars(matrix)
         forge_config = update_matrix(forge_config, matrix)
 
+        fast_finish = textwrap.dedent("""\
+            ({get_fast_finish_script} | \\
+                python - -v --ci "travis" "${{TRAVIS_REPO_SLUG}}" "${{TRAVIS_BUILD_NUMBER}}" "${{TRAVIS_PULL_REQUEST}}") || exit 1
+        """)
+        get_fast_finish_script = ""
+
+        # If the recipe supplies its own conda-forge-build-setup script,
+        # we use it instead of the global one.
+        cfbs_fpath = os.path.join(forge_dir, 'recipe',
+                                  'ff_ci_pr_build.py')
+        if os.path.exists(cfbs_fpath):
+            get_fast_finish_script += "cat {recipe_dir}/ff_ci_pr_build.py".format(recipe_dir=forge_config["recipe_dir"])
+        else:
+            get_fast_finish_script += "curl https://raw.githubusercontent.com/conda-forge/conda-forge-build-setup-feedstock/master/recipe/ff_ci_pr_build.py"
+
+        fast_finish = fast_finish.format(
+            get_fast_finish_script=get_fast_finish_script
+        )
+
+        fast_finish = fast_finish.strip()
+        fast_finish = fast_finish.replace("\n", "\n      ")
+
+        forge_config['fast_finish'] = fast_finish
+
         build_setup = ""
 
         # If the recipe supplies its own conda-forge-build-setup script,
@@ -385,7 +418,6 @@ def render_appveyor(jinja_env, forge_config, forge_dir):
                 arch_env = MatrixCaseEnvVar('TARGET_ARCH', arch)
                 full_matrix.extend([arch_env] + list(case)
                                    for case in cases_not_skipped)
-
     matrix = sorted(full_matrix, key=sort_without_target_arch)
 
     target_fname = os.path.join(forge_dir, 'appveyor.yml')
@@ -425,6 +457,34 @@ def render_appveyor(jinja_env, forge_config, forge_dir):
 
         forge_config = update_matrix(forge_config, matrix)
 
+        get_fast_finish_script = ""
+        fast_finish_script = ""
+        fast_finish = textwrap.dedent("""\
+            {get_fast_finish_script}
+            {fast_finish_script} -v --ci "appveyor" "%APPVEYOR_ACCOUNT_NAME%/%APPVEYOR_PROJECT_SLUG%" "%APPVEYOR_BUILD_NUMBER%" "%APPVEYOR_PULL_REQUEST_NUMBER%"
+        """)
+
+        # If the recipe supplies its own conda-forge-build-setup script,
+        # we use it instead of the global one.
+        cfbs_fpath = os.path.join(forge_dir, 'recipe',
+                                  'ff_ci_pr_build.py')
+        if os.path.exists(cfbs_fpath):
+            fast_finish_script += "{recipe_dir}\\ff_ci_pr_build".format(recipe_dir=forge_config["recipe_dir"])
+        else:
+            get_fast_finish_script += "curl https://raw.githubusercontent.com/conda-forge/conda-forge-build-setup-feedstock/master/recipe/ff_ci_pr_build.py > ff_ci_pr_build.py"
+            fast_finish_script += "ff_ci_pr_build"
+            fast_finish += "del {fast_finish_script}.py"
+
+        fast_finish = fast_finish.format(
+            get_fast_finish_script=get_fast_finish_script,
+            fast_finish_script=fast_finish_script,
+        )
+
+        fast_finish = fast_finish.strip()
+        fast_finish = fast_finish.replace("\n", "\n        ")
+
+        forge_config['fast_finish'] = fast_finish
+
         build_setup = ""
 
         # If the recipe supplies its own conda-forge-build-setup script,
@@ -478,12 +538,12 @@ def update_matrix(forge_config, new_matrix):
 
 def prepare_matrix_for_env_vars(matrix):
     """
-    Turns a matrix with environment variables and pakages into a matrix of
+    Turns a matrix with environment variables and packages into a matrix of
     just environment variables. The package variables are prefixed with CONDA,
-    and special cases such as Python and Numpy are handled.
+    and special cases such as Python, NumPy, and R are handled.
 
     """
-    special_conda_vars = {'python': 'CONDA_PY', 'numpy': 'CONDA_NPY'}
+    special_conda_vars = {'python': 'CONDA_PY', 'numpy': 'CONDA_NPY', 'r-base': 'CONDA_R'}
     env_matrix = []
     for case in matrix:
         new_case = []
@@ -495,7 +555,8 @@ def prepare_matrix_for_env_vars(matrix):
                 name, value = item
                 if name in special_conda_vars:
                     name = special_conda_vars[name]
-                    value = str(value).replace('.', '')
+                    if name.upper() != 'CONDA_R':
+                        value = str(value).replace('.', '')
                 else:
                     name = 'CONDA_' + name.upper()
                 new_case.append((name, value))
@@ -563,7 +624,7 @@ def compute_build_matrix(meta, existing_matrix=None, channel_sources=tuple()):
     index = conda.api.get_index(channel_urls=channel_sources,
                                 platform=meta_config(meta).subdir)
     mtx = special_case_version_matrix(meta, index)
-    mtx = list(filter_cases(mtx, ['python >=2.7,<3|>=3.5', 'numpy >=1.11']))
+    mtx = list(filter_cases(mtx, ['python >=2.7,<3|>=3.5', 'numpy >=1.11', 'r-base >=3.3.2']))
     if existing_matrix:
         mtx = [tuple(mtx_case) + tuple(MatrixCaseEnvVar(*item) for item in case)
                for case in sorted(existing_matrix)
@@ -641,7 +702,6 @@ def main(forge_file_directory):
 
     copy_feedstock_content(forge_dir)
 
-    render_run_docker_build(env, config, forge_dir)
     render_circle(env, config, forge_dir)
     render_travis(env, config, forge_dir)
     render_appveyor(env, config, forge_dir)
